
/*******************************************************************************
DISCLAIMER

THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.

WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
INTELLECTUAL PROPERTY INFRINGIMENT.

TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.

BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
THIS SOFTWARE.
BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.

IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
File:		userif.c
Project:	SK_S7G2_MotorControl
Revision:	1.0
Date:		01-06-16 (DD-MM-YYYY)
Author:		Andrea Ghirardello
Contents:	User interface protocol
*******************************************************************************/

#define _USERIF_C	// insert define BEFORE including header files

#include "globdef.h"
#include "i_ges_eqp.h"
#include ".\library\BLE\r_RY7011.h"

// serial communication protocol
#define	IND_LEN		(   0 )	// string length (1 byte) index
#define	IND_SEL		(   1 )	// master/slave identifier (1 byte) index
#define	IND_ADS		(   2 )	// station address (1 byte) index
#define	IND_COD		(   3 )	// operation code (1 byte) index
#define IND_ADD		(   4 )	// data address (1 byte) index
#define	IND_NDA		(   5 )	// data number (1 byte) index
#define	IND_IDA		(   6 )	// data start index
#define	LEN_MIN		(   5 )	// shortest frame length, crc included
#define	LEN_DEL		(   2 )	// additional characters, data excluded
#define	LEN_MAX		( 255 )	// longest frame length, crc included (must stay in a byte!)
#define	BUF_LEN		( 256 )	// rx/tx buffer dim. (includes a final 0 terminator, not transmitted)
#define	MAX_NBD		(LEN_MAX - LEN_MIN - LEN_DEL)	// max number of byte data ((255-5-2)      = 248)
#define	MAX_NWD		(MAX_NBD >> 1 )					// max number of word data (((255-5-2)>>1) = 124)
#define	MAX_NLD		(MAX_NWD >> 1 )					// max number of long data (((255-5-2)>>2) =  62)
#define SEL_MAS		'?'		// ( 0x3F =  63 ) master frame identification character (question)
#define SEL_SLA		'!'		// ( 0x21 =  33 ) slave frame identification character (answer OK)
#define SEL_NOK		'#'		// ( 0x23 =  35 ) slave frame identification character (answer NOK)
#define CCO			'c'		// ( 0x63 =  99 ) Check Com operation code
#define NCO			'C'		// ( 0x43 =  67 ) Check Com operation code (answer)
#define XCO			'd'		// ( 0x64 = 100 ) Check Com operation code (answer, programs with auto-tuning)
#define YCO			'D'		// ( 0x44 =  68 ) Check Com operation code (answer, programs with auto-tuning and new buffers)
#define CC1			'e'		// ( 0x65 = 101 ) Check Com operation code (answer, new programs with floating point parameters and variables)
#define RBO			'b'		// ( 0x62 =  98 ) Read Byte operation code
#define WBO			'B'		// ( 0x42 =  66 ) Write Byte operation code
#define RBM			'h'		// ( 0x68 = 104 ) Read Byte M operation code (read in measurement samples vector)
#define RWO			'w'		// ( 0x77 = 119 ) Read Word operation code
#define WWO			'W'		// ( 0x57 =  87 ) Write Word operation code
#define RW2			'y'		// ( 0x79 = 121 ) Read Word 2 operation code (parameter minimum values)
#define RW3			'z'		// ( 0x7A = 122 ) Read Word 3 operation code (parameter maximum values)
#define RW4			'j'		// ( 0x6A = 106 ) Read Word 4 operation code (parameter default values)
#define RWM			'k'		// ( 0x6B = 107 ) Read Word M operation code (read in measurement samples vector)
#define RLO			'l'		// ( 0x6C = 108 ) Read Long operation code (read in ram table)
#define WLO			'L'		// ( 0x4C =  76 ) Write Long operation code (write in ram table)
#define RL1			'p'		// ( 0x70 = 112 ) Read Long 1 operation code (read in parameters vector)
#define WL1			'P'		// ( 0x50 =  80 ) Write Long 1 operation code (write in parameters vector)
#define RL2			'Y'		// ( 0x59 =  89 ) Read Long 2 operation code (parameter minimum values)
#define RL3			'Z'		// ( 0x5A =  90 ) Read Long 3 operation code (parameter default values)
#define RL4			'J'		// ( 0x4A =  74 ) Read Long 4 operation code (parameter maximum values)

#define	SLA_IDN		(   0 )	// slave identification number (point-to-point communication)

// IMPORTANT NOTE: depending on the parameters type and of the ram_tab elements type (16 or 32 bits),
// to comply with the protocol requirements, the following relations MUST be verified:
// if type is 32bit then: NUM_PAR<MAX_NLD, N_RAM_READ<MAX_NLD, N_RAM_WRITE<MAX_NLD
// if type is 16bit then: NUM_PAR<MAX_NWD, N_RAM_READ<MAX_NWD, N_RAM_WRITE<MAX_NWD
// in every case: MAX_SAM_CNT<MAX_NWD

typedef union
{
	uint16_t	word;
	struct
	{
		uint8_t	l;
		uint8_t	h;
	}	byte;
} 	u16_bytevec_t;

typedef union
{
	uint32_t		longw;
/*	struct
	{
		uint16_t	wl;
		uint16_t	wh;
	}	word; */
	struct
	{
		uint8_t		bll;
		uint8_t		blh;
		uint8_t		bhl;
		uint8_t		bhh;
	}	byte;
} 	u32_bytevec_t;

// constants
const uint8_t
	crc_tab[256] =	// DOW CRC table
	{
		  0,  94, 188, 226,  97,  63, 221, 131, 194, 156, 126,  32, 163, 253,  31,  65,
		157, 195,  33, 127, 252, 162,  64,  30,  95,   1, 227, 189,  62,  96, 130, 220,
		 35, 125, 159, 193,  66,  28, 254, 160, 225, 191,  93,   3, 128, 222,  60,  98,
		190, 224,   2,  92, 223, 129,  99,  61, 124,  34, 192, 158,  29,  67, 161, 255,
		 70,  24, 250, 164,  39, 121, 155, 197, 132, 218,  56, 102, 229, 187,  89,   7,
		219, 133, 103,  57, 186, 228,   6,  88,  25,  71, 165, 251, 120,  38, 196, 154,
		101,  59, 217, 135,   4,  90, 184, 230, 167, 249,  27,  69, 198, 152, 122,  36,
		248, 166,  68,  26, 153, 199,  37, 123,  58, 100, 134, 216,  91,   5, 231, 185,
		140, 210,  48, 110, 237, 179,  81,  15,  78,  16, 242, 172,  47, 113, 147, 205,
		 17,  79, 173, 243, 112,  46, 204, 146, 211, 141, 111,  49, 178, 236,  14,  80,
		175, 241,  19,  77, 206, 144, 114,  44, 109,  51, 209, 143,  12,  82, 176, 238,
		 50, 108, 142, 208,  83,  13, 239, 177, 240, 174,  76,  18, 145, 207,  45, 115,
		202, 148, 118,  40, 171, 245,  23,  73,   8,  86, 180, 234, 105,  55, 213, 139,
		 87,   9, 235, 181,  54, 104, 138, 212, 149, 203,  41, 119, 244, 170,  72,  22,
		233, 183,  85,  11, 136, 214,  52, 106,  43, 117, 151, 201,  74,  20, 246, 168,
		116,  42, 200, 150,  21,  75, 169, 247, 182, 232,  10,  84, 215, 137, 107,  53
	};

uint16_t
	u16_rtx_num,
	u16_sam_cnt;

extern uint16_t
	rtx232_num,
	add_par,
	dta_par,
	sam_cnt;

bool_t newMsgReceived = FALSE;
uint8_t* newRxMsg;
uint8_t newRxMsgLen;

// JS Test input
#define  MAX_FRAME	74
#define  END        0x05
uint8_t  MstrCommand_rxMsg[256];
uint8_t  Mirror_txMsg[256];
uint8_t  MstrMsg_index;
uint8_t  frame_length;
uint8_t  frame_part;
uint8_t  total_frame;
uint8_t  offset;
uint8_t  copy_orglength;
uint8_t  master_rcvd_ok = FALSE;
uint8_t  msg_ready      = FALSE;
uint16_t delay;
uint8_t dumm_s;
uint8_t tst_value = 20;
uint8_t tst_mesage[7] = {0x07, 0x3f, 0x00, 0x6B, 0x00, 0x78, 0x30};
// JS TEST END

static bool_t cbRxIndication(uint8_t* rxMsg, uint8_t  length)
{
	newMsgReceived = true;
	newRxMsg = rxMsg;
	newRxMsgLen = length;
	return TRUE;
}

static bool_t cbTxIndication(uint8_t* MtxMsg, uint8_t  length)//(uint8_t* txMsg, uint8_t  length)
{
	if(MtxMsg[0] == DEVMGMT_STATUS_OK)//(txMsg[0] == DEVMGMT_STATUS_OK)
	{
		master_rcvd_ok = TRUE;
		//UI_LED_Toggle(LED_3);
	}
	else
	{
		master_rcvd_ok = FALSE;
	}
	return TRUE;
}

/******************************************************************************
Function:		b2w
Description:	2 bytes vector conversion in a word
Input:			two bytes vector *v
Output:			the short obtained by the union of the two bytes
******************************************************************************/
static uint16_t b2w(uint8_t *u8v)
{
	u16_bytevec_t u16w;

	u16w.byte.h = u8v[0];
	u16w.byte.l = u8v[1];
	return(u16w.word);
}

/******************************************************************************
Function:		w2b
Description:	1 word conversion in a 2 bytes vector
Input:			the word to convert and the address of the result vector
Output:			the address of the result vector
Modifies:		the vector content
******************************************************************************/
static uint8_t *w2b(uint16_t u16w, uint8_t *u8v)
{
	u16_bytevec_t u16x;

	u16x.word = u16w;
	u8v[0] = u16x.byte.h;
	u8v[1] = u16x.byte.l;
	return(u8v);
}

/******************************************************************************
Function:		b2l
Description:	4 bytes vector conversion in a long
Input:			four bytes vector *v
Output:			the long obtained by the union of the four bytes
******************************************************************************/
static uint32_t b2l(uint8_t *u8v)
{
	u32_bytevec_t u32l;

	u32l.byte.bhh = u8v[0];
	u32l.byte.bhl = u8v[1];
	u32l.byte.blh = u8v[2];
	u32l.byte.bll = u8v[3];
	return(u32l.longw);
}

/******************************************************************************
Function:		l2b
Description:	1 long conversion in a 4 bytes vector
Input:			the long to convert and the address of the result vector
Output:			the address of the result vector
Modifies:		the vector content
******************************************************************************/
static uint8_t *l2b(uint32_t u32l, uint8_t *u8v)
{
	u32_bytevec_t u32x;

	u32x.longw = u32l;
	u8v[0] = u32x.byte.bhh;
	u8v[1] = u32x.byte.bhl;
	u8v[2] = u32x.byte.blh;
	u8v[3] = u32x.byte.bll;
	return(u8v);
}

/******************************************************************************
Function:		cal_crc
Description:	DOW CRC calculation
Input:			the number of bytes to be taken into account,
				the vector containing these bytes
Output:			CRC
Note:			it uses the DOW CRC calculation table defined before
******************************************************************************/
static uint8_t cal_crc(uint8_t u8n, uint8_t *u8v)
{
	uint8_t	u8i, u8cks = 0;

	for(u8i = 0; u8i < u8n; u8i++)
	{
		u8cks = crc_tab[u8cks ^ u8v[u8i]];
	}
	return(u8cks);
}

/******************************************************************************
Function: 		BLE_Init
Description:	serial communication init
Input:			nothing
Output:			nothing
Modifies:		internal variables involved in serial communication
******************************************************************************/
void BLE_Init(void)
{
	R_RY7011_Init();
	R_RY7011_RegisterClient(cbRxIndication, cbTxIndication);
}

/* decode function */
/******************************************************************************
Function:		decode_rx
Description:
Input:			received characters vector
Output:			nothing
Modifies		received characters vector
Note:
******************************************************************************/
void decode_rx(uint8_t *u8v)
{
	uint16_t u16add, u16nda, u16i;

	u16_rtx_num = u8v[IND_LEN];

	// errors in rx
	if(LEN_MIN > u16_rtx_num)
	{
		u8v[0] = 0;
		return;
	}

	// no answer cases
	if((SEL_MAS != u8v[IND_SEL]) || (SLA_IDN != u8v[IND_ADS]))
	{
		u8v[0] = 0;
		return;
	}
	if(u8v[u16_rtx_num - 1] != cal_crc((u16_rtx_num - 1), u8v))
	{
		u8v[0] = 0;
		return;
	}

	// operation codes select and answer coding
	u16add = u8v[IND_ADD];					// will be used only if valid!
	u16nda = u8v[IND_NDA];					// will be used only if valid!
	u8v[IND_SEL] = SEL_SLA;
    u16_rtx_num = LEN_MIN;					// frame lenght
	switch(u8v[IND_COD])
	{
		case CCO:	// check
		case NCO:	// check
		case XCO:	// check (programs with auto tuning)
		case YCO:	// check (programs with auto tuning and new buffers)
		case CC1:	// check (new programs with auto tuning, new buffers and float parameters and variables)
			u8v[IND_COD] = YCO;	// answer of programs with auto tuning and new buffers
			break;
		case RWO:	// answer coding
			if(NUM_PAR > u16add)	// reading from parameters table
			{
				if((NUM_PAR < u16nda) || (u16add > (NUM_PAR - u16nda)))
				{
					u8v[IND_SEL] = SEL_NOK;
					break;
				}
				for(u16i = 0; u16i < u16nda; u16i++)
				{
					w2b(p_vec[u16add + u16i], &(u8v[IND_IDA + (u16i << 1)]));
				}
				u16_rtx_num = u16_rtx_num + (u16nda << 1) + LEN_DEL;	// frame lenght
			}
			else					// reading from ram table
			{
				u16add = u16add - NUM_PAR;
				if((N_RAM_READ < u16nda) || (u16add > (N_RAM_READ - u16nda)))
				{
					u8v[IND_SEL] = SEL_NOK;
					break;
				}
				for(u16i = 0; u16i < u16nda; u16i++)
				{
					w2b(UIF_R.ram_tab[u16add + u16i], &(u8v[IND_IDA + (u16i << 1)]));
				}
				u16_rtx_num = u16_rtx_num + (u16nda << 1) + LEN_DEL;	// frame lenght
			}
			break;
		case WWO:	// answer coding
			if(NUM_PAR > u16add)	// writing in parameters table
			{
				if((NUM_PAR < u16nda) || (u16add > (NUM_PAR - u16nda)))
				{
					u8v[IND_SEL] = SEL_NOK;
					break;
				}
				for(u16i = 0; u16i < u16nda; u16i++)
				{
					GE_SetPar(b2w(&(u8v[IND_IDA + (u16i << 1)])), u16add + u16i);
				}
			}
			else					// writing in ram table
			{
				u16add = u16add - NUM_PAR;
				if((N_RAM_WRITE < u16nda) || (u16add > (N_RAM_WRITE - u16nda)))
				{
					u8v[IND_SEL] = SEL_NOK;
					break;
				}
				for(u16i = 0; u16i < u16nda; u16i++)
				{
					UIF_W.ram_tab[u16add + u16i] = b2w(&(u8v[IND_IDA + (u16i << 1)]));
				}
			}
			break;
		case RW2:	// reading parameters minimum values
			if((NUM_PAR <= u16add) || (u16nda > (NUM_PAR - u16add)))
			{
				u8v[IND_SEL] = SEL_NOK;
				break;
			}
			for(u16i = 0; u16i < u16nda; u16i++)
			{
				w2b(p_min[u16add + u16i], &(u8v[IND_IDA + (u16i << 1)]));
			}
			u16_rtx_num = u16_rtx_num + (u16nda << 1) + LEN_DEL;	// frame lenght
			break;
		case RW3:	// reading parameters maximum values
			if((NUM_PAR <= u16add) || (u16nda > (NUM_PAR - u16add)))
			{
				u8v[IND_SEL] = SEL_NOK;
				break;
			}
			for(u16i = 0; u16i < u16nda; u16i++)
			{
				w2b(p_max[u16add + u16i], &(u8v[IND_IDA + (u16i << 1)]));
			}
			u16_rtx_num = u16_rtx_num + (u16nda << 1) + LEN_DEL;	// frame lenght
			break;
		case RW4:	// reading parameters table default values
			if((NUM_PAR <= u16add) || (u16nda > (NUM_PAR - u16add)))
			{
				u8v[IND_SEL] = SEL_NOK;
				break;
			}
			for(u16i = 0; u16i < u16nda; u16i++)
			{
				w2b(p_def[u16add + u16i], &(u8v[IND_IDA + (u16i << 1)]));
			}
		    u16_rtx_num = u16_rtx_num + (u16nda << 1) + LEN_DEL;	// frame lenght
			break;
		case RWM:	// reading measurement samples
			if((MAX_SAM_CNT <= u16add) || (u16nda > (MAX_SAM_CNT - u16add)) || (u16nda > MAX_NWD))
			{
				u8v[IND_SEL] = SEL_NOK;
				break;
			}
			if(0 == u16add)
			{
				SET_RBUSY
				u16_sam_cnt = 0;
			}
			for(u16i = 0; u16i < u16nda; u16i++)
			{
				w2b(au16_rbuf[u16add + u16i], &(u8v[IND_IDA + (u16i << 1)]));
			}
			u16_sam_cnt = u16_sam_cnt + u16nda;
			if(MAX_SAM_CNT <= u16_sam_cnt)
			{
				RES_RBUSY
			}
			u16_rtx_num = u16_rtx_num + (u16nda << 1) + LEN_DEL;	// frame lenght
			break;
		default:
			u8v[IND_SEL] = SEL_NOK;
			break;
	}

	// crc calc. and answer transmission
	u8v[IND_LEN] = u16_rtx_num;
	u8v[u16_rtx_num - 1] = cal_crc(u16_rtx_num - 1, u8v);
	u8v[u16_rtx_num] = 0;			// ("C" string)
	return;

}

/******************************************************************************
Function:       decode_ble
Description:
Input:          received characters vector
Output:         nothing
Modifies        received characters vector
Note:
******************************************************************************/
void decode_ble(uint8_t *u8v)
{
    uint16_t u16add, u16nda, u16i;

    u16_rtx_num = u8v[IND_LEN];

/*
    // errors in rx
    if(LEN_MIN > u16_rtx_num)
    {
        u8v[0] = 0;
        return;
    }

    // no answer cases
    if((SEL_MAS != u8v[IND_SEL]) || (SLA_IDN != u8v[IND_ADS]))
    {
        u8v[0] = 0;
        return;
    }
    if(u8v[u16_rtx_num - 1] != cal_crc((u16_rtx_num - 1), u8v))
    {
        u8v[0] = 0;
        return;
    }
*/

    // operation codes select and answer coding
    u16add = u8v[IND_ADD];                  // will be used only if valid!
    u16nda = u8v[IND_NDA];                  // will be used only if valid!
    u8v[IND_SEL] = SEL_SLA;
    u16_rtx_num = LEN_MIN;                  // frame lenght
    switch(u8v[IND_COD])
    {
        case CCO:   // check
        case NCO:   // check
        case XCO:   // check (programs with auto tuning)
        case YCO:   // check (programs with auto tuning and new buffers)
        case CC1:   // check (new programs with auto tuning, new buffers and float parameters and variables)
            u8v[IND_COD] = YCO; // answer of programs with auto tuning and new buffers
            break;
        case RWO:   // answer coding
            if(NUM_PAR > u16add)    // reading from parameters table
            {
                if((NUM_PAR < u16nda) || (u16add > (NUM_PAR - u16nda)))
                {
                    u8v[IND_SEL] = SEL_NOK;
                    break;
                }
                for(u16i = 0; u16i < u16nda; u16i++)
                {
                    w2b(p_vec[u16add + u16i], &(u8v[IND_IDA + (u16i << 1)]));
                }
                u16_rtx_num = u16_rtx_num + (u16nda << 1) + LEN_DEL;    // frame lenght
            }
            else                    // reading from ram table
            {
                u16add = u16add - NUM_PAR;
                if((N_RAM_READ < u16nda) || (u16add > (N_RAM_READ - u16nda)))
                {
                    u8v[IND_SEL] = SEL_NOK;
                    break;
                }
                for(u16i = 0; u16i < u16nda; u16i++)
                {
                    w2b(UIF_R.ram_tab[u16add + u16i], &(u8v[IND_IDA + (u16i << 1)]));
                }
                u16_rtx_num = u16_rtx_num + (u16nda << 1) + LEN_DEL;    // frame lenght
            }
            break;
        case WWO:   // answer coding
            if(NUM_PAR > u16add)    // writing in parameters table
            {
                if((NUM_PAR < u16nda) || (u16add > (NUM_PAR - u16nda)))
                {
                    u8v[IND_SEL] = SEL_NOK;
                    break;
                }
                for(u16i = 0; u16i < u16nda; u16i++)
                {
                    GE_SetPar(b2w(&(u8v[IND_IDA + (u16i << 1)])), u16add + u16i);
                }
            }
            else                    // writing in ram table
            {
                u16add = u16add - NUM_PAR;
                if((N_RAM_WRITE < u16nda) || (u16add > (N_RAM_WRITE - u16nda)))
                {
                    u8v[IND_SEL] = SEL_NOK;
                    break;
                }
                for(u16i = 0; u16i < u16nda; u16i++)
                {
                    UIF_W.ram_tab[u16add + u16i] = b2w(&(u8v[IND_IDA + (u16i << 1)]));
                }
            }
            break;
        case RW2:   // reading parameters minimum values
            if((NUM_PAR <= u16add) || (u16nda > (NUM_PAR - u16add)))
            {
                u8v[IND_SEL] = SEL_NOK;
                break;
            }
            for(u16i = 0; u16i < u16nda; u16i++)
            {
                w2b(p_min[u16add + u16i], &(u8v[IND_IDA + (u16i << 1)]));
            }
            u16_rtx_num = u16_rtx_num + (u16nda << 1) + LEN_DEL;    // frame lenght
            break;
        case RW3:   // reading parameters maximum values
            if((NUM_PAR <= u16add) || (u16nda > (NUM_PAR - u16add)))
            {
                u8v[IND_SEL] = SEL_NOK;
                break;
            }
            for(u16i = 0; u16i < u16nda; u16i++)
            {
                w2b(p_max[u16add + u16i], &(u8v[IND_IDA + (u16i << 1)]));
            }
            u16_rtx_num = u16_rtx_num + (u16nda << 1) + LEN_DEL;    // frame lenght
            break;
        case RW4:   // reading parameters table default values
            if((NUM_PAR <= u16add) || (u16nda > (NUM_PAR - u16add)))
            {
                u8v[IND_SEL] = SEL_NOK;
                break;
            }
            for(u16i = 0; u16i < u16nda; u16i++)
            {
                w2b(p_def[u16add + u16i], &(u8v[IND_IDA + (u16i << 1)]));
            }
            u16_rtx_num = u16_rtx_num + (u16nda << 1) + LEN_DEL;    // frame lenght
            break;
        case RWM:   // reading measurement samples
            if((MAX_SAM_CNT <= u16add) || (u16nda > (MAX_SAM_CNT - u16add)) || (u16nda > MAX_NWD))
            {
                u8v[IND_SEL] = SEL_NOK;
                break;
            }
            if(0 == u16add)
            {
                SET_RBUSY
                u16_sam_cnt = 0;
            }
            for(u16i = 0; u16i < u16nda; u16i++)
            {
                w2b(au16_rbuf[u16add + u16i], &(u8v[IND_IDA + (u16i << 1)]));
            }
            u16_sam_cnt = u16_sam_cnt + u16nda;
            if(MAX_SAM_CNT <= u16_sam_cnt)
            {
                RES_RBUSY
            }
            u16_rtx_num = u16_rtx_num + (u16nda << 1) + LEN_DEL;    // frame lenght
            break;
        default:
            u8v[IND_SEL] = SEL_NOK;
            break;
    }

    // crc calc. and answer transmission
    u8v[IND_LEN] = u16_rtx_num;
    u8v[u16_rtx_num - 1] = cal_crc(u16_rtx_num - 1, u8v);
    u8v[u16_rtx_num] = 0;           // ("C" string)
    return;

}

/******************************************************************************
Function:		BLE_Management
Description:	serial communication management
Input:			nothing
Output:			nothing
Modifies:		rx/tx buffer, internal flags
Note:			to be called in the main loop;
				the slave is always waiting for commands (rx); when a command
				is received, it is decoded and, if this makes sense, executed;
				if the command is correctly received or if there are errors
				in the data or in the addresses, the slave sends an answer.
******************************************************************************/
//void UI_RTX232_Management()
void BLE_Management()
{
	//R_RY7011_Task();

	if(frame_part == 0x00) // Check first if some frames have still to be send...
	{
		if(newMsgReceived) //otherwise No Frames open so check if new msg received
		{
			for (MstrMsg_index = 0; MstrMsg_index < newRxMsgLen; MstrMsg_index++)
				{
					MstrCommand_rxMsg[MstrMsg_index] = *newRxMsg; //copy the data into
					newRxMsg++; 								  //next data address
				}
			newRxMsg -= newRxMsgLen; 							  //set back the start address MUST BE WHEN NO MANIPULATION USED

//            decode_rx(MstrCommand_rxMsg);
            decode_ble(MstrCommand_rxMsg);
/* test
            for(u16_rtx_num = 1; u16_rtx_num < 70; u16_rtx_num++)
            {
                MstrCommand_rxMsg[u16_rtx_num] = u16_rtx_num;
            }
            MstrCommand_rxMsg[0] = 70;
fine test */
            rtx232_num = MstrCommand_rxMsg[IND_LEN];
//            MstrCommand_rxMsg[rtx232_num] = 0xFF;

		}//if(newMsgReceived) END
/*
		//Check if frame is > 74 Bytes
		if(rtx232_num > MAX_FRAME)
		{
			frame_length = MAX_FRAME;
			copy_orglength = rtx232_num;
			total_frame = copy_orglength/MAX_FRAME;
			total_frame++;
			total_frame = total_frame<<4;
			frame_part = 0x01;
			newMsgReceived = FALSE;
		}// if(rtx232_num > MAX_FRAME) END
		else */ if(newMsgReceived)
		{
		  R_RY7011_SendBLEMessage(MstrCommand_rxMsg, rtx232_num);
		  newMsgReceived = FALSE;
		}//else end

	}//if(frame_part == 0x00) END

/*
		if((copy_orglength != 0x00)&&(frame_part!=0x00))
		{
				switch(frame_part)
				{
					case 0x01:
								for(dumm_s = 0; dumm_s < frame_length; dumm_s++)
								{
									Mirror_txMsg[dumm_s] = MstrCommand_rxMsg[dumm_s+frame_length];
								}
								MstrCommand_rxMsg[frame_length-1] = (total_frame + frame_part);	//Indicate the frame part
								R_RY7011_SendBLEMessage(MstrCommand_rxMsg, frame_length); //slip sends from 1 to frame_length
								frame_part++;
								copy_orglength -= frame_length;
								if(master_rcvd_ok) master_rcvd_ok = FALSE;  // Dies kann auch im n�chsten case abgefragt werden
					break;
					case 0x02:
								if(copy_orglength > MAX_FRAME)
									{
										copy_orglength -= frame_length;
									}
								else
									{
										frame_length = copy_orglength; //Start address of the next frame
									}

								if(!master_rcvd_ok)
								{
									offset = MAX_FRAME; //Start address of the next frame
									(MstrCommand_rxMsg)[(offset+frame_length)-1] = (total_frame + frame_part);
									R_RY7011_SendBLEMessage(MstrCommand_rxMsg + offset, frame_length);
									if(frame_length == copy_orglength) frame_part = 0x00;
									else frame_part++;
									master_rcvd_ok = FALSE;
								}
					break;
					case 0x03:
								if(copy_orglength > MAX_FRAME) copy_orglength -= (frame_length);
								else frame_length = copy_orglength;

								if(!master_rcvd_ok)
								{
									offset = 2*MAX_FRAME; //Start address of the next frame
									(MstrCommand_rxMsg)[(offset + frame_length)-1] = (total_frame + frame_part);
									R_RY7011_SendBLEMessage(MstrCommand_rxMsg+offset, frame_length);
									if(frame_length == copy_orglength) frame_part = 0x00;
									else frame_part++;
									master_rcvd_ok = FALSE;
								}
					break;
					case 0x04:
								if(copy_orglength > MAX_FRAME)
									{
										copy_orglength -= frame_length;
										MstrCommand_rxMsg[IND_SEL] = SEL_NOK;
										R_RY7011_SendBLEMessage(MstrCommand_rxMsg, 5); //HAS TO BE CHECKED
										break;
									}
								else
									{
										frame_length = copy_orglength;
									}
								offset = (3*MAX_FRAME);//Start address of the next frame
								if(!master_rcvd_ok)
								{
									(MstrCommand_rxMsg)[(offset+frame_length)] =(total_frame + frame_part);
									R_RY7011_SendBLEMessage(MstrCommand_rxMsg + offset, frame_length+1);
									master_rcvd_ok = FALSE;
									if(frame_length == copy_orglength) frame_part = 0x00;
									else frame_part++;
								}
					break;

					default:
					 		//ERROR HANDLER NEEDED HERE...
					break;
				}//switch end

			}//if(copy_orglength != 0x00) END
			if (frame_part == 0x00)
				frame_part = frame_length = total_frame = copy_orglength = offset = 0x00;
			return;
	//}
	//return;

*/
}

// EOF userif.c
